@@ -23,7 +23,6 @@ ggv_prefs_dialog_apply(GgvPrefsDialog *dlg)
         GtkWidget *active;
         gint n, i;
 
-        gtk_widget_set_sensitive(dlg->ok, FALSE);
         gtk_widget_set_sensitive(dlg->apply, FALSE);
 
         gtk_gs_defaults_set_interpreter_cmd
@@ -129,7 +128,7 @@ ggv_prefs_dialog_cancel_clicked(GtkWidget *widget, gpointer *data)
 }
 
 static gboolean
-ggv_prefs_dialog_delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data)
+ggv_prefs_dialog_delete_event(GtkWidget *widget, GdkEventAny *event)
 {
         gtk_widget_hide(widget);
 
@@ -542,7 +541,7 @@ ggv_prefs_dialog_new()
         table = gtk_table_new(8, 1, FALSE);
 
         /* show side panel */
-        dlg->sidebar = gtk_check_button_new_with_label(_("Show side panel by default"));
+        dlg->sidebar = gtk_check_button_new_with_label(_("Show side panel"));
         gtk_table_attach(GTK_TABLE(table), dlg->sidebar,
                          0, 1, 0, 1,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -563,7 +562,7 @@ ggv_prefs_dialog_new()
         gtk_widget_show(dlg->right_panel);
 
         /* show menubar */
-        dlg->mbar = gtk_check_button_new_with_label(_("Show menubar by default"));
+        dlg->mbar = gtk_check_button_new_with_label(_("Show menubar"));
         gtk_table_attach(GTK_TABLE(table), dlg->mbar,
                          0, 1, 2, 3,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -573,7 +572,7 @@ ggv_prefs_dialog_new()
                            G_CALLBACK(prefs_dlg_change_handler), dlg);
         gtk_widget_show(dlg->mbar);
 	/* show toolbar */
-        dlg->toolbar = gtk_check_button_new_with_label(_("Show toolbar by default"));
+        dlg->toolbar = gtk_check_button_new_with_label(_("Show toolbar"));
         gtk_table_attach(GTK_TABLE(table), dlg->toolbar,
                          0, 1, 3, 4,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -606,7 +605,7 @@ ggv_prefs_dialog_new()
         }
                                    
         /* show statusbar */
-        dlg->sbar = gtk_check_button_new_with_label(_("Show statusbar by default"));
+        dlg->sbar = gtk_check_button_new_with_label(_("Show statusbar"));
         gtk_table_attach(GTK_TABLE(table), dlg->sbar,
                          0, 1, 5, 6,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
